c     Created by Mahdi Esmaily Moghadam 5-23-2011
c     Please report any problem to mesmaily@ucsd.edu, memt63@yahoo.com

c This subroutine initializes the parameters, you need to read the
c comments and specify them carefuly
c--------------------------------------------------------------------
c This is an example of Heart model connected to a cylinder via two
c inductances. There in this example both coupled surfaces are Dirichlet

      SUBROUTINE INITIALIZE(nTimeStep)

      USE COM
      USE model_para
      LOGICAl ierr

      INTEGER nTimeStep

      REAL*8, ALLOCATABLE :: tZeroX(:)

      INTENT(OUT) nTimeStep

c********************************************************************
c For instance if pressure in 3D solver is in cgs and here mmHg
c pConv=1334=1.334D3, also if flowrate in 3D solver is mm^3/s and
c here is mL/s qConv=1000=1D3. In the case both solver are using same
c unites you can set these two conversion coefficients to 1D0
      pConv = 1.333D3
      qConv = -1D0

c Only when all the surfaces of you model are coupled with NeumannSrfs
c you may set this to .TRUE.
      pCorr = .FALSE.
      qCorr = .FALSE.

c********************************************************************
c Block for your inputs

c These two value should match to that of defined in solver.inp
c Since pressures are prescribed at the inlet, the inlet is treated as a Neumann surface
      nDirichletSrfs = 0
      nNeumannSrfs   = 1
c Number of unknowns that you need inside your lumped parameter network
      nUnknowns      = 1
c Number of time step between N and N+alpha, for higher values this code
c would be more accurate and more costly
      nTimeStep = 2000

c Number of parameters to be printed in AllData file (the first
c nUnknowns columns are by default X)
      nXprint = 3

c--------------------------------------------------------------------
c You don't need to change this part                                 !
      ALLOCATE (tZeroX(nUnknowns), srfToXdPtr(nDirichletSrfs))       !
      ALLOCATE (srfToXPtr(nNeumannSrfs))                             !
      tZeroX = 0D0                                                   !
c--------------------------------------------------------------------

c Value of your unknown at time equal to zero (This is going to be used
c ONLY when you are initiating simulation)
c set initial X (pressure at the inlet) to be 15mmHg
      tZeroX = (/1.8D0 /)
c       tZeroX = (/1.333D1 /)
c--------------------------------------------------------------------
c You don't need to change this part                                 !
      INQUIRE (FILE='InitialData', EXIST=ierr)                       !
      IF (.NOT.ierr) THEN                                            !
         PRINT *, 'Initializing unknowns in LPM'                     !
         OPEN (1, FILE='InitialData',STATUS='NEW',FORM='UNFORMATTED')!
         WRITE (1) 0D0                                               !
         DO i=1, nUnknowns                                           !
            WRITE (1) tZeroX(i)                                      !
         END DO                                                      !
         CLOSE(1)                                                    !
      END IF                                                         !
c--------------------------------------------------------------------

c Surface to X pointer: this defines which Unknown corresponds to which
c suface in "List of Neumann Surfaces" inside solver.inp
c For example, if you have "List of Neumann Surfaces= 2 8 4 ...."
c and you set "srfToXPtr = (/5,3,9,.../)"
C this means X(5) corresponds to surface 2, X(3) <-> surface 8,
c and X(9) <-> surface 4
c      srfToXPtr = (/3/)
c This is exactly the same for srfToXdPtr only srfToXdPtr is for Dirichlet
c surfaces
         srfToXPtr = (/1/)
c        srfToXdPtr = (/1/)

c        INQUIRE (FILE='PV_status', EXIST=ierr)
c        IF (.NOT.ierr) THEN
c        OPEN(1, FILE='PV_status', STATUS='NEW',ACTION='WRITE')
c        WRITE(1,*) 0.0,0.0
c        CLOSE(1)
c        ENDIF

      END SUBROUTINE INITIALIZE



c####################################################################
c Here you should find the f_i=dx_i/dt, based on the following parameters:
c  current x_i:                   x(i)
c  Current time:                  t
c  Flowrates of Neumann faces:    Q(i)
c  Pressures of Dirichlet faces:  P(i)

      SUBROUTINE FINDF(t,told, x, f, Q, P)

      USE COM
      USE model_para


      IMPLICIT NONE

      REAL*8 t, told,x(nUnknowns), f(nUnknowns), Q(nNeumannSrfs),
     2   P(nDirichletSrfs),tau,tt
      INTEGER i
      REAL*8, dimension(20) :: Pinput,tinput

      INTENT(IN) t,told, Q
      INTENT(INOUT) x
      INTENT(OUT) f

      tinput= (/ 0.0,
     &     0.031000,
     &     0.062000,
     &     0.093000,
     &     0.124000,
     &     0.155000,
     &     0.186000,
     &     0.216000,
     &     0.247000,
     &     0.278000,
     &     0.309000,
     &     0.340000,
     &     0.371000,
     &     0.402000,
     &     0.433000,
     &     0.464000,
     &     0.495000,
     &     0.526000,
     &     0.557000,
     &     0.588000 /)
      Pinput=(/ 1.868200,
     &     4.964600,
     &     17.746800,
     &     24.753800,
     &     25.966400,
     &     22.933900,
     &     19.437300,
     &     16.290900,
     &     13.330800,
     &     9.321900,
     &     4.545400,
     &     0.091400,
     &     0.355300,
     &     1.357600,
     &     1.485200,
     &     0.933200,
     &     0.379500,
     &     0.414000,
     &     0.740000,
     &     1.036700 /)

c To prescribe a pressure waveform, update x(1) directly by a linear interpolation and set f(1)=0
       f(1)=0.0
       tt=dmod(t,tinput(size(tinput)))
       DO i=1,size(tinput)-1
         IF (tt .GT. tinput(i) .AND. tt .LE. tinput(i+1) ) THEN
            x(1)=Pinput(i)+(Pinput(i+1)-Pinput(i))/
     &   (tinput(i+1)-tinput(i))*(tt-tinput(i))
            EXIT
         ENDIF
       ENDDO

c Print additional data into AllData, unknowns X(n) are always written into the first n columns of AllData then followed by additional variables Xprint defined by users.
      Xprint(1) = t
      Xprint(2) = x(1)
      Xprint(3) = Q(1)



      RETURN
      END SUBROUTINE FINDF







